#!/usr/bin/python
"""
'Naputus' is a simple memory and reflex game.  Four buttons in the
bottom of the screen are lighted in random order at increasing speed.
User should press the corresponding functions keys (F1-F4) in the
same order.
"""


import random
import pygame

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.image: print 'Warning, images disabled'
if not pygame.mixer: print 'Warning, sound disabled'

# Important constant definitions for Pygame, 
# in this example: used to setup the "Double 
# Buffered" Display...
from pygame.locals import *


class Naputus:
    
    def __init__(self, wd=640, ht=480):
	"initialize the game, load images, sounds etc"
	
	# initialise SDL environment
	pygame.init()
	
	# setup screen
	self.screen = pygame.display.set_mode((wd,ht), DOUBLEBUF)
	
	# Make the screen surface black (R,G,B)
	self.screen.fill((0,0,0))
	
	# setup font
	font = pygame.font.Font(None, 80)
	text = font.render("Naputus v1", 1, (0, 0, 255))
	pos = text.get_rect()
	pos.top = 20
	pos.centerx = self.screen.get_rect().centerx
	self.screen.blit(text, pos)

	# score showing settings
	self.score_y = pos.bottom + 20
	self.font = pygame.font.Font(None, 40)

	# initial score
	self.hiscore = 0
	self.draw_score(self.hiscore, 1)
	self.draw_score(0, 0)

	# Now display the screen buffer
	pygame.display.flip()

	print "Loading:"
	
	# load game sounds
	pygame.mixer.init(22050)
	print "pop.wav..."
	self.sound_pop = pygame.mixer.Sound('pop.wav')
	print "monkey.wav..."
	self.sound_monkey = pygame.mixer.Sound('monkey.wav')
	
	# load button images
	self.init_buttons(wd, ht, 160)
	
	# window settings
	pygame.display.set_caption('Naputus v1')
	icon = pygame.transform.scale(self.button[0], (32, 32))
	pygame.display.set_icon(icon)
	pygame.mouse.set_visible(0)
	
	# show the buttons
	for i in range(4):
	    self.draw_button(i)

	# and their explanations
	self.draw_keys()
    

    def init_buttons(self, wd, ht, size):
	"Load images and initialize their positions"
	
	img_off = (wd - size * 4) / 5
	img_y = ht - size

	# load images
	self.button = []
	self.pos = []

	for i in range(1, 9):
	    name = "%d.jpg" % i
	    print name + "..."

	    # load and make sure it's same format as screen
	    img = pygame.image.load(name).convert()
	    self.button.append(img)
	    
	    # position images (bright and dim versions)
	    # evenly on the bottom of the screen.
	    pos = img.get_rect()
	    pos.top = img_y
	    pos.left = (i-1)/2 * (size + img_off) + img_off
	    self.pos.append(pos)


    def draw_keys(self):
	"draw key names on top of buttons"

	for i in range(4):
	    idx = i * 2
	    pos = self.pos[idx]
	    text = self.font.render("F%d" % (i+1), 1, (0, 0, 255))
	    textpos = text.get_rect()
	    textpos.top = pos.top - textpos.height - 10
	    textpos.centerx = pos.centerx
	    self.screen.blit(text, textpos)
	    pygame.display.update(textpos)
	
	    
    def draw_button(self, num, state=0):
	"""draw_button(num, state)
	num -- The button number (0-3)
	state -- 0 for dimmed, 1 for hightlighted
	"""
	if num < 0 or num > 3:
	    print "ERROR: num value %d wrong" % num
	idx = 2*num + state
	pos = self.pos[idx]
	button = self.button[idx]

	self.screen.blit(button, pos)
	pygame.display.update(pos)

	    
    def draw_score(self, score, hi):
	"draws the current score"
	if hi:
	    text = "hi-score"
	else:
	    text = "score"
	area = self.font.render("%s: %d" % (text, score), 1, (255, 0, 0))
	pos = area.get_rect()
	if hi:
	    pos.top = self.score_y
	else:
	    pos.top = self.score_y + pos.height + 10
	pos.centerx = self.screen.get_rect().centerx
	fillpos = pos.inflate(20, 0)
	self.screen.fill((0,0, 0), fillpos)
	self.screen.blit(area, pos)
	pygame.display.update(fillpos)

	    
    def presskey(self):
	"color cycle 'press a key' message and wait key press"

	for event in pygame.event.get():
	    pass
	
	g = 10
	b = 240
	amount = 10
	while 1:
	    if g < 10 or g > 240:
		amount = -amount
	    b -= amount
	    g += amount
	    area = self.font.render("press a key to start", 1, (b, g, 255))
	    pos = area.get_rect()
	    pos.top = self.score_y + 2 * (pos.height + 10)
	    pos.centerx = self.screen.get_rect().centerx
	    self.screen.blit(area, pos)
	    pygame.display.update(pos)
	    pygame.time.wait(100)

	    for event in pygame.event.get():
		if event.type == QUIT:
		    return 0
		if event.type == KEYUP:
		    if event.key == K_ESCAPE:
			return 0
		    else:
			self.screen.fill((0,0,0), pos)
			pygame.display.update(pos)
			return 1

	    
    def run(self):
	"The game main loop"

	pygame.event.set_allowed([QUIT, KEYUP])

	# wait until user starts game
	if not self.presskey():
	    return 0

	score = 0
	self.draw_score(score, 0)
	
	keys = { K_F1: 0, K_F2: 1, K_F3: 2, K_F4: 3 }
	random.seed()
	values = []
	value = 0

	skip = 0
	count = 0
	sleep = 800
	# a little wait
	pygame.time.wait(2000)
	timer = pygame.time.get_ticks() + sleep

	while sleep > 0:
	    # too slow reactions
	    if len(values) > 5:
		break
	    # new button
	    value = random.randrange(0,4)
	    values.append(value)

	    # brighten new button
	    self.draw_button(value, 1)
	    pygame.time.wait(250)

	    # play sound
	    self.sound_pop.play()

	    # dim last button
	    self.draw_button(value, 0)

	    # wait (gets shorter)
	    count += 1
	    if count > skip:
		sleep -=  30
		skip += 1
		count = 0
	    wait = timer - pygame.time.get_ticks()
	    if wait > 0:
		# wait until we've done our timeframe
		pygame.time.wait(wait)
	    timer = pygame.time.get_ticks() + sleep

	    # check events
	    for event in pygame.event.get():
		if event.type == QUIT:
		    return 0
		
		if event.type == KEYUP:
		    if event.key == K_ESCAPE:
			return 0
		    if not values:
			# key pressed when no button was ON
			break
		    if keys.has_key(event.key):
			#print keys[event.key], values[0]
			if keys[event.key] == values[0]:
			    score += 1
			    self.draw_score(score, 0)
			    # consume given button
			    values = values[1:]
			else:
			    # wrong key, end game
			    sleep = 0
			    break
	
	# end game
	pygame.time.wait(250)
	self.sound_monkey.play()
	# update highscore
	if score > self.hiscore:
	    self.hiscore = score
	    self.draw_score(self.hiscore, 1)
	pygame.time.wait(1600)
	return 1


# So we can run straight from the CLI...
if __name__ == '__main__':
    game = Naputus()
    while 1:
	if not game.run():
	    break

